@@ -86,7 +86,7 @@ class Firewall(object):
             self.ebtables_enabled = True
             self.ipset_backend = ipset.ipset()
             self.ipset_enabled = True
-            self.ipset_supported_types = [ ]
+            self.ipset_supported_types = IPSET_TYPES
             self.nftables_backend = nftables.nftables(self)
             self.nftables_enabled = True
 
@@ -105,12 +105,13 @@ class Firewall(object):
         self.__init_vars()
 
     def __repr__(self):
-        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % \
+        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % \
             (self.__class__, self.ip4tables_enabled, self.ip6tables_enabled,
              self.ebtables_enabled, self._state, self._panic,
              self._default_zone, self._module_refcount, self._marks,
-             self.cleanup_on_exit, self.ipv6_rpfilter_enabled,
-             self.ipset_enabled, self._individual_calls, self._log_denied)
+             self.cleanup_on_exit, self.cleanup_modules_on_exit,
+             self.ipv6_rpfilter_enabled, self.ipset_enabled,
+             self._individual_calls, self._log_denied)
 
     def __init_vars(self):
         self._state = "INIT"
@@ -120,6 +121,7 @@ class Firewall(object):
         self._marks = [ ]
         # fallback settings will be overloaded by firewalld.conf
         self.cleanup_on_exit = config.FALLBACK_CLEANUP_ON_EXIT
+        self.cleanup_modules_on_exit = config.FALLBACK_CLEANUP_MODULES_ON_EXIT
         self.ipv6_rpfilter_enabled = config.FALLBACK_IPV6_RPFILTER
         self._individual_calls = config.FALLBACK_INDIVIDUAL_CALLS
         self._log_denied = config.FALLBACK_LOG_DENIED
@@ -131,18 +133,18 @@ class Firewall(object):
     def _check_tables(self):
         # check if iptables, ip6tables and ebtables are usable, else disable
         if self.ip4tables_enabled and \
-           "filter" not in self.get_backend_by_ipv("ipv4").get_available_tables():
-            log.warning("iptables not usable, disabling IPv4 firewall.")
+           "filter" not in self.ip4tables_backend.get_available_tables():
+            log.info1("iptables is not usable.")
             self.ip4tables_enabled = False
 
         if self.ip6tables_enabled and \
-           "filter" not in self.get_backend_by_ipv("ipv6").get_available_tables():
-            log.warning("ip6tables not usable, disabling IPv6 firewall.")
+           "filter" not in self.ip6tables_backend.get_available_tables():
+            log.info1("ip6tables is not usable.")
             self.ip6tables_enabled = False
 
         if self.ebtables_enabled and \
-           "filter" not in self.get_backend_by_ipv("eb").get_available_tables():
-            log.warning("ebtables not usable, disabling ethernet bridge firewall.")
+           "filter" not in self.ebtables_backend.get_available_tables():
+            log.info1("ebtables is not usable.")
             self.ebtables_enabled = False
 
         # is there at least support for ipv4 or ipv6
@@ -155,10 +157,13 @@ class Firewall(object):
         try:
             self.ipset_backend.set_list()
         except ValueError:
-            log.warning("ipset not usable, disabling ipset usage in firewall.")
-            # ipset is not usable, no supported types
+            if self.nftables_enabled:
+                log.info1("ipset not usable, disabling ipset usage in firewall. Other set backends (nftables) remain usable.")
+            else:
+                log.warning("ipset not usable, disabling ipset usage in firewall.")
+                self.ipset_supported_types = [ ]
+            # ipset is not usable
             self.ipset_enabled = False
-            self.ipset_supported_types = [ ]
         else:
             # ipset is usable, get all supported types
             self.ipset_supported_types = self.ipset_backend.set_supported_types()
@@ -169,8 +174,12 @@ class Firewall(object):
                 log.warning("iptables-restore is missing, using "
                             "individual calls for IPv4 firewall.")
             else:
-                log.warning("iptables-restore and iptables are missing, "
-                            "disabling IPv4 firewall.")
+                if self.nftables_enabled:
+                    log.info1("iptables-restore and iptables are missing, "
+                              "IPv4 direct rules won't be usable.")
+                else:
+                    log.warning("iptables-restore and iptables are missing, "
+                                "disabling IPv4 firewall.")
                 self.ip4tables_enabled = False
         if self.nftables_enabled:
             self.ipv4_supported_icmp_types = self.nftables_backend.supported_icmp_types("ipv4")
@@ -185,8 +194,12 @@ class Firewall(object):
                 log.warning("ip6tables-restore is missing, using "
                             "individual calls for IPv6 firewall.")
             else:
-                log.warning("ip6tables-restore and ip6tables are missing, "
-                            "disabling IPv6 firewall.")
+                if self.nftables_enabled:
+                    log.info1("ip6tables-restore and ip6tables are missing, "
+                              "IPv6 direct rules won't be usable.")
+                else:
+                    log.warning("ip6tables-restore and ip6tables are missing, "
+                                "disabling IPv6 firewall.")
                 self.ip6tables_enabled = False
         if self.nftables_enabled:
             self.ipv6_supported_icmp_types = self.nftables_backend.supported_icmp_types("ipv6")
@@ -201,8 +214,12 @@ class Firewall(object):
                 log.warning("ebtables-restore is missing, using "
                             "individual calls for bridge firewall.")
             else:
-                log.warning("ebtables-restore and ebtables are missing, "
-                            "disabling bridge firewall.")
+                if self.nftables_enabled:
+                    log.info1("ebtables-restore and ebtables are missing, "
+                              "eb direct rules won't be usable.")
+                else:
+                    log.warning("ebtables-restore and ebtables are missing, "
+                                "disabling bridge firewall.")
                 self.ebtables_enabled = False
 
         if self.ebtables_enabled and not self._individual_calls and \
@@ -232,6 +249,13 @@ class Firewall(object):
                 log.debug1("CleanupOnExit is set to '%s'",
                            self.cleanup_on_exit)
 
+            if self._firewalld_conf.get("CleanupModulesOnExit"):
+                value = self._firewalld_conf.get("CleanupModulesOnExit")
+                if value is not None and value.lower() in [ "yes", "true" ]:
+                    self.cleanup_modules_on_exit = True
+                log.debug1("CleanupModulesOnExit is set to '%s'",
+                           self.cleanup_modules_on_exit)
+
             if self._firewalld_conf.get("Lockdown"):
                 value = self._firewalld_conf.get("Lockdown")
                 if value is not None and value.lower() in [ "yes", "true" ]:
@@ -291,20 +315,6 @@ class Firewall(object):
                 log.debug1("RFC3964_IPv4 is set to '%s'",
                            self._rfc3964_ipv4)
 
-            if self._firewalld_conf.get("AllowZoneDrifting"):
-                value = self._firewalld_conf.get("AllowZoneDrifting")
-                if value.lower() in [ "no", "false" ]:
-                    self._allow_zone_drifting = False
-                else:
-                    self._allow_zone_drifting = True
-                    if not self._offline:
-                        log.warning("AllowZoneDrifting is enabled. This is considered "
-                                    "an insecure configuration option. It will be "
-                                    "removed in a future release. Please consider "
-                                    "disabling it now.")
-                log.debug1("AllowZoneDrifting is set to '%s'",
-                           self._allow_zone_drifting)
-
         self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
         self._select_firewall_backend(self._firewall_backend)
@@ -423,7 +433,7 @@ class Firewall(object):
         # only needs to be split here if there are conflicting ipset types in
         # exsting ipsets and the configuration in firewalld.
         if (reload and complete_reload) or \
-           (self.ipset_enabled and self.ipset.has_ipsets()):
+           (self.ipset.backends() and self.ipset.has_ipsets()):
             transaction.execute(True)
             transaction.clear()
 
@@ -437,7 +447,7 @@ class Firewall(object):
         transaction.clear()
 
         # apply settings for loaded ipsets while reloading here
-        if self.ipset_enabled and self.ipset.has_ipsets():
+        if (self.ipset.backends()) and self.ipset.has_ipsets():
             log.debug1("Applying ipsets")
             self.ipset.apply_ipsets()
 
@@ -667,11 +677,15 @@ class Firewall(object):
         self.__init_vars()
 
     def stop(self):
-        if self.cleanup_on_exit and not self._offline:
-            self.flush()
-            self.ipset.flush()
-            self.set_policy("ACCEPT")
-            self.modules_backend.unload_firewall_modules()
+        if not self._offline:
+            if self.cleanup_on_exit:
+                self.flush()
+                self.ipset.flush()
+                self.set_policy("ACCEPT")
+
+            if self.cleanup_modules_on_exit:
+                log.debug1('Unloading firewall kernel modules')
+                self.modules_backend.unload_firewall_modules()
 
         self.cleanup()
 
