@@ -24,7 +24,8 @@
 __all__ = [ "FirewallIPSet" ]
 
 from firewall.core.logger import log
-from firewall.core.ipset import remove_default_create_options as rm_def_cr_opts
+from firewall.core.ipset import remove_default_create_options as rm_def_cr_opts, \
+                                normalize_ipset_entry, check_entry_overlaps_existing
 from firewall.core.io.ipset import IPSet
 from firewall import errors
 from firewall.errors import FirewallError
@@ -189,11 +190,13 @@ class FirewallIPSet(object):
 
     def add_entry(self, name, entry):
         obj = self.get_ipset(name, applied=True)
+        entry = normalize_ipset_entry(entry)
 
         IPSet.check_entry(entry, obj.options, obj.type)
         if entry in obj.entries:
             raise FirewallError(errors.ALREADY_ENABLED,
                                 "'%s' already is in '%s'" % (entry, name))
+        check_entry_overlaps_existing(entry, obj.entries)
 
         try:
             for backend in self.backends():
@@ -201,13 +204,13 @@ class FirewallIPSet(object):
         except Exception as msg:
             raise FirewallError(errors.COMMAND_FAILED, msg)
         else:
-            if "timeout" not in obj.options or obj.options["timeout"] == "0" \
-               and entry not in obj.entries:
+            if "timeout" not in obj.options or obj.options["timeout"] == "0":
                 # no entries visible for ipsets with timeout
                 obj.entries.append(entry)
 
     def remove_entry(self, name, entry):
         obj = self.get_ipset(name, applied=True)
+        entry = normalize_ipset_entry(entry)
 
         # no entry check for removal
         if entry not in obj.entries:
@@ -219,13 +222,13 @@ class FirewallIPSet(object):
         except Exception as msg:
             raise FirewallError(errors.COMMAND_FAILED, msg)
         else:
-            if "timeout" not in obj.options or obj.options["timeout"] == "0" \
-               and entry not in obj.entries:
+            if "timeout" not in obj.options or obj.options["timeout"] == "0":
                 # no entries visible for ipsets with timeout
                 obj.entries.remove(entry)
 
     def query_entry(self, name, entry):
         obj = self.get_ipset(name, applied=True)
+        entry = normalize_ipset_entry(entry)
         if "timeout" in obj.options and obj.options["timeout"] != "0":
             # no entries visible for ipsets with timeout
             raise FirewallError(errors.IPSET_WITH_TIMEOUT, name)
@@ -239,6 +242,12 @@ class FirewallIPSet(object):
     def set_entries(self, name, entries):
         obj = self.get_ipset(name, applied=True)
 
+        _entries = set()
+        for _entry in entries:
+            check_entry_overlaps_existing(_entry, _entries)
+            _entries.add(normalize_ipset_entry(_entry))
+        entries = list(_entries)
+
         for entry in entries:
             IPSet.check_entry(entry, obj.options, obj.type)
         if "timeout" not in obj.options or obj.options["timeout"] == "0":
