@@ -26,7 +26,7 @@ import shutil
 
 from firewall import config
 from firewall.fw_types import LastUpdatedOrderedDict
-from firewall.functions import splitArgs, joinArgs, u2b_if_py2
+from firewall.functions import splitArgs, joinArgs
 from firewall.core.io.io_object import IO_Object, IO_Object_ContentHandler, \
     IO_Object_XMLGenerator
 from firewall.core.logger import log
@@ -58,8 +58,7 @@ class direct_ContentHandler(IO_Object_ContentHandler):
             ipv = attrs["ipv"]
             table = attrs["table"]
             chain = attrs["chain"]
-            self.item.add_chain(u2b_if_py2(ipv), u2b_if_py2(table),
-                                u2b_if_py2(chain))
+            self.item.add_chain(ipv, table, chain)
 
         elif name == "rule":
             if not self.direct:
@@ -77,15 +76,14 @@ class direct_ContentHandler(IO_Object_ContentHandler):
                 log.error("Parse Error: %s is not a valid priority" %
                           attrs["priority"])
                 return
-            self._rule = [ u2b_if_py2(ipv), u2b_if_py2(table),
-                           u2b_if_py2(chain), priority ]
+            self._rule = [ ipv, table, chain, priority ]
 
         elif name == "passthrough":
             if not self.direct:
                 log.error("Parse Error: command outside of direct")
                 return
             ipv = attrs["ipv"]
-            self._passthrough = [ u2b_if_py2(ipv) ]
+            self._passthrough = [ ipv ]
 
         else:
             log.error('Unknown XML element %s' % name)
@@ -97,8 +95,7 @@ class direct_ContentHandler(IO_Object_ContentHandler):
         if name == "rule":
             if self._element:
                 # add arguments
-                self._rule.append([ u2b_if_py2(x)
-                                    for x in splitArgs(self._element) ])
+                self._rule.append(splitArgs(self._element))
                 self.item.add_rule(*self._rule)
             else:
                 log.error("Error: rule does not have any arguments, ignoring.")
@@ -106,8 +103,7 @@ class direct_ContentHandler(IO_Object_ContentHandler):
         elif name == "passthrough":
             if self._element:
                 # add arguments
-                self._passthrough.append([ u2b_if_py2(x)
-                                           for x in splitArgs(self._element) ])
+                self._passthrough.append(splitArgs(self._element))
                 self.item.add_passthrough(*self._passthrough)
             else:
                 log.error("Error: passthrough does not have any arguments, " +
@@ -142,7 +138,7 @@ class Direct(IO_Object):
         self.rules = LastUpdatedOrderedDict()
         self.passthroughs = LastUpdatedOrderedDict()
 
-    def _check_config(self, conf, item):
+    def _check_config(self, conf, item, all_conf):
         pass
         # check arg lists
 
@@ -360,9 +356,11 @@ class Direct(IO_Object):
         handler = direct_ContentHandler(self)
         parser = sax.make_parser()
         parser.setContentHandler(handler)
-        with open(self.filename, "r") as f:
+        with open(self.filename, "rb") as f:
+            source = sax.InputSource(None)
+            source.setByteStream(f)
             try:
-                parser.parse(f)
+                parser.parse(source)
             except sax.SAXParseException as msg:
                 raise FirewallError(errors.INVALID_TYPE,
                                     "Not a valid file: %s" % \
