@@ -77,6 +77,22 @@ def common_startElement(obj, name, attrs):
             else:
                 log.warning("Protocol '%s' already set, ignoring.",
                             attrs["value"])
+    
+    elif name == "tcp-mss-clamp":
+        if obj._rule:
+            if obj._rule.element:            
+                log.warning("Invalid rule: More than one element in rule '%s', ignoring.",
+                            str(obj._rule))
+                obj._rule_error = True
+                return True
+            _value="pmtu"
+            if "value" in attrs:
+                _value = attrs["value"]
+            obj._rule.element = rich.Rich_Tcp_Mss_Clamp(_value)
+        else:
+            log.warning("Invalid rule: tcp-mss-clamp '%s' outside of rule",
+                        attrs["value"])
+
     elif name == "icmp-block":
         if obj._rule:
             if obj._rule.element:
@@ -186,11 +202,18 @@ def common_startElement(obj, name, attrs):
                         str(obj._rule))
             return True
         invert = False
+        address = None
+        if "address" in attrs:
+            address = attrs["address"]
+        ipset = None
+        if "ipset" in attrs:
+            ipset = attrs["ipset"]
         if "invert" in attrs and \
                 attrs["invert"].lower() in [ "yes", "true" ]:
             invert = True
-        obj._rule.destination = rich.Rich_Destination(attrs["address"],
-                                                       invert)
+        obj._rule.destination = rich.Rich_Destination(address,
+                                                      ipset,
+                                                      invert)
 
     elif name in [ "accept", "reject", "drop", "mark" ]:
         if not obj._rule:
@@ -447,7 +470,11 @@ def common_writer(obj, handler):
 
         # destination
         if rule.destination:
-            attrs = { "address": rule.destination.addr }
+            attrs = { }
+            if rule.destination.addr:
+                attrs["address"] = rule.destination.addr
+            if rule.destination.ipset:
+                attrs["ipset"] = rule.destination.ipset
             if rule.destination.invert:
                 attrs["invert"] = "True"
             handler.ignorableWhitespace("    ")
@@ -469,6 +496,9 @@ def common_writer(obj, handler):
             elif type(rule.element) == rich.Rich_Protocol:
                 element = "protocol"
                 attrs["value"] = rule.element.value
+            elif type(rule.element) == rich.Rich_Tcp_Mss_Clamp:
+                element = "tcp-mss-clamp"
+                attrs["value"] = rule.element.value
             elif type(rule.element) == rich.Rich_Masquerade:
                 element = "masquerade"
             elif type(rule.element) == rich.Rich_IcmpBlock:
@@ -607,7 +637,7 @@ class Policy(IO_Object):
         "forward-port": [ "port", "protocol" ],
         "rule": None,
         "source": None,
-        "destination": [ "address" ],
+        "destination": None,
         "protocol": [ "value" ],
         "source-port": [ "port", "protocol" ],
         "log":  None,
@@ -625,9 +655,10 @@ class Policy(IO_Object):
         "forward-port": [ "to-port", "to-addr" ],
         "rule": [ "family", "priority" ],
         "source": [ "address", "mac", "invert", "family", "ipset" ],
-        "destination": [ "invert" ],
+        "destination": [ "address", "invert", "ipset" ],
         "log": [ "prefix", "level" ],
         "reject": [ "type" ],
+        "tcp-mss-clamp": [ "value" ],
         }
 
     def __init__(self):
