@@ -25,12 +25,9 @@ import xml.sax as sax
 import os
 import io
 import shutil
-import copy
-from collections import OrderedDict
 
 from firewall import config
-from firewall.functions import u2b_if_py2
-from firewall.core.io.io_object import PY2, IO_Object, \
+from firewall.core.io.io_object import IO_Object, \
     IO_Object_ContentHandler, IO_Object_XMLGenerator, check_port, \
     check_tcpudp, check_protocol, check_address
 from firewall.core.logger import log
@@ -80,34 +77,6 @@ class Service(IO_Object):
         self.includes = [ ]
         self.helpers = [ ]
 
-    def import_config(self, conf):
-        self.check_config(conf)
-
-        for key in conf:
-            if not hasattr(self, key):
-                raise FirewallError(errors.UNKNOWN_ERROR, "Internal error. '{}' is not a valid attribute".format(key))
-            if isinstance(conf[key], list):
-                # maintain list order while removing duplicates
-                setattr(self, key, list(OrderedDict.fromkeys(copy.deepcopy(conf[key]))))
-            else:
-                setattr(self, key, copy.deepcopy(conf[key]))
-
-    def export_config(self):
-        conf = {}
-        type_formats = dict([(x[0], x[1]) for x in self.IMPORT_EXPORT_STRUCTURE])
-        for key in type_formats:
-            if getattr(self, key) or isinstance(getattr(self, key), bool):
-                conf[key] = copy.deepcopy(getattr(self, key))
-        return conf
-
-    def check_config(self, conf):
-        type_formats = dict([(x[0], x[1]) for x in self.IMPORT_EXPORT_STRUCTURE])
-        for key in conf:
-            if key not in [x for (x,y) in self.IMPORT_EXPORT_STRUCTURE]:
-                raise FirewallError(errors.INVALID_OPTION, "service option '{}' is not valid".format(key))
-            self._check_config_structure(conf[key], type_formats[key])
-            self._check_config(conf[key], key)
-
     def cleanup(self):
         self.version = ""
         self.short = ""
@@ -120,23 +89,7 @@ class Service(IO_Object):
         del self.includes[:]
         del self.helpers[:]
 
-    def encode_strings(self):
-        """ HACK. I haven't been able to make sax parser return
-            strings encoded (because of python 2) instead of in unicode.
-            Get rid of it once we throw out python 2 support."""
-        self.version = u2b_if_py2(self.version)
-        self.short = u2b_if_py2(self.short)
-        self.description = u2b_if_py2(self.description)
-        self.ports = [(u2b_if_py2(po),u2b_if_py2(pr)) for (po,pr) in self.ports]
-        self.modules = [u2b_if_py2(m) for m in self.modules]
-        self.destination = {u2b_if_py2(k):u2b_if_py2(v) for k,v in self.destination.items()}
-        self.protocols = [u2b_if_py2(pr) for pr in self.protocols]
-        self.source_ports = [(u2b_if_py2(po),u2b_if_py2(pr)) for (po,pr)
-                             in self.source_ports]
-        self.includes = [u2b_if_py2(s) for s in self.includes]
-        self.helpers = [u2b_if_py2(s) for s in self.helpers]
-
-    def _check_config(self, config, item):
+    def _check_config(self, config, item, all_config):
         if item == "ports":
             for port in config:
                 if port[0] != "":
@@ -281,8 +234,6 @@ def service_reader(filename, path):
                                 msg.getException())
     del handler
     del parser
-    if PY2:
-        service.encode_strings()
     return service
 
 def service_writer(service, path=None):
