@@ -24,7 +24,7 @@ __all__ = [ "Rich_Source", "Rich_Destination", "Rich_Service", "Rich_Port",
             "Rich_IcmpType",
             "Rich_SourcePort", "Rich_ForwardPort", "Rich_Log", "Rich_Audit",
             "Rich_Accept", "Rich_Reject", "Rich_Drop", "Rich_Mark",
-            "Rich_Limit", "Rich_Rule" ]
+            "Rich_Limit", "Rich_Rule", "Rich_Tcp_Mss_Clamp" ]
 
 from firewall import functions
 from firewall.core.ipset import check_ipset_name
@@ -63,13 +63,27 @@ class Rich_Source(object):
                                 "no address, mac and ipset")
 
 class Rich_Destination(object):
-    def __init__(self, addr, invert=False):
+    def __init__(self, addr, ipset, invert=False):
         self.addr = addr
+        if self.addr == "":
+            self.addr = None
+        self.ipset = ipset
+        if self.ipset == "":
+            self.ipset = None
         self.invert = invert
+        if self.addr is None and self.ipset is None:
+            raise FirewallError(errors.INVALID_RULE,
+                                "no address and ipset")
 
     def __str__(self):
-        return 'destination %saddress="%s"' % ("not " if self.invert else "",
-                                               self.addr)
+        ret = 'destination%s ' % (" NOT" if self.invert else "")
+        if self.addr is not None:
+            return ret + 'address="%s"' % self.addr
+        elif self.ipset is not None:
+            return ret + 'ipset="%s"' % self.ipset
+        else:
+            raise FirewallError(errors.INVALID_RULE,
+                                "no address and ipset")
 
 class Rich_Service(object):
     def __init__(self, name):
@@ -119,6 +133,13 @@ class Rich_IcmpType(object):
     def __str__(self):
         return 'icmp-type name="%s"' % (self.name)
 
+class Rich_Tcp_Mss_Clamp(object):
+    def __init__(self, value):
+        self.value = value
+
+    def __str__(self):
+        return 'tcp-mss-clamp value="%s"' % (self.value)
+
 class Rich_ForwardPort(object):
     def __init__(self, port, protocol, to_port, to_address):
         self.port = port
@@ -342,7 +363,7 @@ class Rich_Rule(object):
                 if element in ['rule', 'source', 'destination', 'protocol',
                                'service', 'port', 'icmp-block', 'icmp-type', 'masquerade',
                                'forward-port', 'source-port', 'log', 'audit',
-                               'accept', 'drop', 'reject', 'mark', 'limit', 'not', 'NOT', 'EOL']:
+                               'accept', 'drop', 'reject', 'mark', 'limit', 'not', 'NOT', 'EOL', 'tcp-mss-clamp']:
                     if element == 'source' and self.source:
                         raise FirewallError(errors.INVALID_RULE, "more than one 'source' element")
                     elif element == 'destination' and self.destination:
@@ -404,12 +425,12 @@ class Rich_Rule(object):
                     attrs.clear()
                     index = index -1 # return token to input
             elif in_element == 'destination':
-                if attr_name in ['address', 'invert']:
+                if attr_name in ['address', 'ipset', 'invert']:
                     attrs[attr_name] = attr_value
                 elif element in ['not', 'NOT']:
                     attrs['invert'] = True
                 else:
-                    self.destination = Rich_Destination(attrs.get('address'), attrs.get('invert'))
+                    self.destination = Rich_Destination(attrs.get('address'), attrs.get('ipset'), attrs.get('invert', False))
                     in_elements.pop() # destination
                     attrs.clear()
                     index = index -1 # return token to input
@@ -419,6 +440,14 @@ class Rich_Rule(object):
                     in_elements.pop() # protocol
                 else:
                     raise FirewallError(errors.INVALID_RULE, "invalid 'protocol' element")
+            elif in_element == 'tcp-mss-clamp':
+                if attr_name == 'value':
+                    attrs[attr_name] = attr_value
+                else:
+                    self.element = Rich_Tcp_Mss_Clamp(attrs.get('value'))
+                    in_elements.pop()
+                    attrs.clear()
+                    index = index -1
             elif in_element == 'service':
                 if attr_name == 'name':
                     self.element = Rich_Service(attr_value)
@@ -555,7 +584,8 @@ class Rich_Rule(object):
 
         if type(self.element) not in [ Rich_IcmpBlock,
                                        Rich_ForwardPort,
-                                       Rich_Masquerade ]:
+                                       Rich_Masquerade,
+                                       Rich_Tcp_Mss_Clamp ]:
             if self.log is None and self.audit is None and \
                     self.action is None:
                 raise FirewallError(errors.INVALID_RULE, "no action, no log, no audit")
@@ -587,12 +617,20 @@ class Rich_Rule(object):
 
         # destination
         if self.destination is not None:
-            if self.family is None:
-                raise FirewallError(errors.INVALID_FAMILY)
-            if self.destination.addr is None or \
-                    not functions.check_address(self.family,
-                                                self.destination.addr):
-                raise FirewallError(errors.INVALID_ADDR, str(self.destination.addr))
+            if self.destination.addr is not None:
+                if self.family is None:
+                    raise FirewallError(errors.INVALID_FAMILY)
+                if self.destination.ipset is not None:
+                    raise FirewallError(errors.INVALID_DESTINATION, "address and ipset")
+                if not functions.check_address(self.family, self.destination.addr):
+                    raise FirewallError(errors.INVALID_ADDR, str(self.destination.addr))
+
+            elif self.destination.ipset is not None:
+                if not check_ipset_name(self.destination.ipset):
+                    raise FirewallError(errors.INVALID_IPSET, str(self.destination.ipset))
+
+            else:
+                raise FirewallError(errors.INVALID_RULE, "invalid destination")
 
         # service
         if type(self.element) == Rich_Service:
@@ -663,6 +701,14 @@ class Rich_Rule(object):
             if self.element.protocol not in [ "tcp", "udp", "sctp", "dccp" ]:
                 raise FirewallError(errors.INVALID_PROTOCOL, self.element.protocol)
 
+        # tcp-mss-clamp
+        elif type(self.element) == Rich_Tcp_Mss_Clamp:
+            if self.action is not None:
+                raise FirewallError(errors.INVALID_RULE, "tcp-mss-clamp and %s are mutually exclusive" % self.action)
+            if self.element.value:
+                if not functions.checkTcpMssClamp(self.element.value):
+                    raise FirewallError(errors.INVALID_RULE, self.element.value)
+
         # other element and not empty?
         elif self.element is not None:
             raise FirewallError(errors.INVALID_RULE, "Unknown element %s" % 
@@ -715,7 +761,7 @@ class Rich_Rule(object):
         if self.action:
             ret += " %s" % self.action
 
-        return (functions.u2b(ret)) if functions.PY2 else ret
+        return ret
 
 
 #class Rich_RawRule(object):
