@@ -18,24 +18,18 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# force use of pygobject3 in python-slip
-from gi.repository import GObject
-import sys
-sys.modules['gobject'] = GObject
-
 import dbus
 import dbus.service
-import slip.dbus
-import slip.dbus.service
 
 from firewall import config
 from firewall.dbus_utils import dbus_to_python, \
     dbus_introspection_prepare_properties, \
     dbus_introspection_add_properties
-from firewall.core.io.service import Service
 from firewall.core.logger import log
+from firewall.server.dbus import DbusServiceObject
 from firewall.server.decorators import handle_exceptions, \
-    dbus_handle_exceptions, dbus_service_method
+    dbus_handle_exceptions, dbus_service_method, \
+    dbus_polkit_require_auth
 from firewall import errors
 from firewall.errors import FirewallError
 
@@ -45,7 +39,7 @@ from firewall.errors import FirewallError
 #
 ############################################################################
 
-class FirewallDConfigService(slip.dbus.service.Object):
+class FirewallDConfigService(DbusServiceObject):
     """FirewallD main class"""
 
     persistent = True
@@ -92,9 +86,8 @@ class FirewallDConfigService(slip.dbus.service.Object):
             return dbus.Boolean(self.obj.builtin)
         else:
             raise dbus.exceptions.DBusException(
-                "org.freedesktop.DBus.Error.AccessDenied: "
-                "Property '%s' isn't exported (or may not exist)" % \
-                property_name)
+                "org.freedesktop.DBus.Error.InvalidArgs: "
+                "Property '%s' does not exist" % property_name)
 
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ss',
                          out_signature='v')
@@ -109,7 +102,7 @@ class FirewallDConfigService(slip.dbus.service.Object):
         if interface_name != config.dbus.DBUS_INTERFACE_CONFIG_SERVICE:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.UnknownInterface: "
-                "FirewallD does not implement %s" % interface_name)
+                "Interface '%s' does not exist" % interface_name)
 
         return self._get_property(property_name)
 
@@ -123,14 +116,14 @@ class FirewallDConfigService(slip.dbus.service.Object):
         if interface_name != config.dbus.DBUS_INTERFACE_CONFIG_SERVICE:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.UnknownInterface: "
-                "FirewallD does not implement %s" % interface_name)
+                "Interface '%s' does not exist" % interface_name)
 
         ret = { }
         for x in [ "name", "filename", "path", "default", "builtin" ]:
             ret[x] = self._get_property(x)
         return dbus.Dictionary(ret, signature="sv")
 
-    @slip.dbus.polkit.require_auth(config.dbus.PK_ACTION_CONFIG)
+    @dbus_polkit_require_auth(config.dbus.PK_ACTION_CONFIG)
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ssv')
     @dbus_handle_exceptions
     def Set(self, interface_name, property_name, new_value, sender=None):
@@ -144,11 +137,11 @@ class FirewallDConfigService(slip.dbus.service.Object):
         if interface_name != config.dbus.DBUS_INTERFACE_CONFIG_SERVICE:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.UnknownInterface: "
-                "FirewallD does not implement %s" % interface_name)
+                "Interface '%s' does not exist" % interface_name)
 
         raise dbus.exceptions.DBusException(
-            "org.freedesktop.DBus.Error.AccessDenied: "
-            "Property '%s' is not settable" % property_name)
+            "org.freedesktop.DBus.Error.PropertyReadOnly: "
+            "Property '%s' is read-only" % property_name)
 
     @dbus.service.signal(dbus.PROPERTIES_IFACE, signature='sa{sv}as')
     def PropertiesChanged(self, interface_name, changed_properties,
@@ -159,7 +152,7 @@ class FirewallDConfigService(slip.dbus.service.Object):
         log.debug1("%s.PropertiesChanged('%s', '%s', '%s')", self._log_prefix,
                    interface_name, changed_properties, invalidated_properties)
 
-    @slip.dbus.polkit.require_auth(config.dbus.PK_ACTION_INFO)
+    @dbus_polkit_require_auth(config.dbus.PK_ACTION_INFO)
     @dbus_service_method(dbus.INTROSPECTABLE_IFACE, out_signature='s')
     @dbus_handle_exceptions
     def Introspect(self, sender=None): # pylint: disable=W0613
@@ -174,7 +167,7 @@ class FirewallDConfigService(slip.dbus.service.Object):
     # S E T T I N G S
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
-                         out_signature=Service.DBUS_SIGNATURE)
+                         out_signature='(sssa(ss)asa{ss}asa(ss))')
     @dbus_handle_exceptions
     def getSettings(self, sender=None): # pylint: disable=W0613
         """get settings for service
@@ -183,7 +176,16 @@ class FirewallDConfigService(slip.dbus.service.Object):
         return self.config.get_service_config(self.obj)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
-                         in_signature=Service.DBUS_SIGNATURE)
+                         out_signature='a{sv}')
+    @dbus_handle_exceptions
+    def getSettings2(self, sender=None):
+        """get settings for service
+        """
+        log.debug1("%s.getSettings2()", self._log_prefix)
+        return self.config.get_service_config_dict(self.obj)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='(sssa(ss)asa{ss}asa(ss))')
     @dbus_handle_exceptions
     def update(self, settings, sender=None):
         """update settings for service
@@ -194,6 +196,16 @@ class FirewallDConfigService(slip.dbus.service.Object):
         self.obj = self.config.set_service_config(self.obj, settings)
         self.Updated(self.obj.name)
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='a{sv}')
+    @dbus_handle_exceptions
+    def update2(self, settings, sender=None):
+        settings = dbus_to_python(settings)
+        log.debug1("%s.update2('...')", self._log_prefix)
+        self.parent.accessCheck(sender)
+        self.obj = self.config.set_service_config_dict(self.obj, settings)
+        self.Updated(self.obj.name)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE)
     @dbus_handle_exceptions
     def loadDefaults(self, sender=None):
@@ -653,3 +665,58 @@ class FirewallDConfigService(slip.dbus.service.Object):
         settings = self.getSettings()
         return (family in settings[5] and
                 address == settings[5][family])
+
+    # includes
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         out_signature='as')
+    @dbus_handle_exceptions
+    def getIncludes(self, sender=None):
+        log.debug1("%s.getIncludes()", self._log_prefix)
+        self.parent.accessCheck(sender)
+        settings = self.config.get_service_config_dict(self.obj)
+        return settings["includes"] if "includes" in settings else []
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='as')
+    @dbus_handle_exceptions
+    def setIncludes(self, includes, sender=None):
+        includes = dbus_to_python(includes, list)
+        log.debug1("%s.setIncludes('%s')", self._log_prefix, includes)
+        self.parent.accessCheck(sender)
+        settings = {"includes": includes[:]}
+        self.config.set_service_config_dict(self.obj, settings)
+        self.Updated(self.obj.name)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='s')
+    @dbus_handle_exceptions
+    def addInclude(self, include, sender=None):
+        include = dbus_to_python(include, str)
+        log.debug1("%s.addInclude('%s')", self._log_prefix, include)
+        self.parent.accessCheck(sender)
+        settings = self.config.get_service_config_dict(self.obj)
+        settings.setdefault("includes", []).append(include)
+        self.config.set_service_config_dict(self.obj, settings)
+        self.Updated(self.obj.name)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='s')
+    @dbus_handle_exceptions
+    def removeInclude(self, include, sender=None):
+        include = dbus_to_python(include, str)
+        log.debug1("%s.removeInclude('%s')", self._log_prefix, include)
+        self.parent.accessCheck(sender)
+        settings = self.config.get_service_config_dict(self.obj)
+        settings["includes"].remove(include)
+        self.config.set_service_config_dict(self.obj, settings)
+        self.Updated(self.obj.name)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_SERVICE,
+                         in_signature='s', out_signature='b')
+    @dbus_handle_exceptions
+    def queryInclude(self, include, sender=None):
+        include = dbus_to_python(include, str)
+        log.debug1("%s.queryInclude('%s')", self._log_prefix, include)
+        settings = self.config.get_service_config_dict(self.obj)
+        return include in settings["includes"] if "includes" in settings else False
